/*
 * A system pager implementation residing in the same task as a program.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <exec/pager.h>
#include <fsserver/resource.h>

#include "pager_object_interface.h"



/* A simple system pager also acting as a region mapper. */

class InternalPager : public ExecPager, public PagerObject, public Resource
{
public:
  explicit InternalPager(address_t start = 0, address_t end = 0);

  /* Server details. */

  virtual ipc_server_default_config_type config();

  virtual void *interface()
  { return static_cast<PagerObject *>(this); }

  /* Resource methods. */

  virtual void close();

  /* Notification methods, implementing PagerObject. */

  virtual long exception(l4_exc_regs_t regs,
                         l4_snd_fpage_t *region);

  virtual long page_fault(l4_umword_t pfa, l4_umword_t pc,
                          l4_snd_fpage_t *region);

  /* Region manager/mapper methods. */

  virtual long attach(address_t *start, address_t size, map_flags_t flags,
                      l4_cap_idx_t ds, address_t offset, unsigned char align);
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
